<?php

declare(strict_types=1);

namespace Drupal\ragapay\PluginForm\OffsiteRedirect;

use Drupal\ragapay\Helper\HashBuilder;
use Drupal\Core\Form\FormStateInterface;
use Drupal\ragapay\Helper\PriceFormatter;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\Methods;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\Operations;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\PayloadFields;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\RagaPayOffsite;
use Drupal\ragapay\Plugin\Commerce\PaymentGateway\ResponseFields;

/**
 * RagaPay offsite form.
 */
class RagaPayOffsiteForm extends PaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    $form = parent::buildConfigurationForm($form, $form_state);
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->getEntity();
    $data = [
      'return' => $form['#return_url'],
      'cancel' => $form['#cancel_url'],
      'total' => $payment->getAmount()->getNumber(),
    ];
    $redirectUrl = $this->getExternalRedirectUrl($payment, $data['return'], $data['cancel']);
    if (!$redirectUrl) {
      $redirectUrl = $data['cancel'];
      throw PaymentGatewayException::createForPayment($payment, 'Could not get the redirect URL.');
    }
    $form = $this->buildRedirectForm($form, $form_state, $redirectUrl, $data);
    return $form;
  }

  /**
   * Get the external redirect URL.
   */
  protected function getExternalRedirectUrl(PaymentInterface $payment, string $successUrl, string $cancelUrl): ?string {
    $client = \Drupal::service('ragapay.client');
    $payload = $this->buildSessionPayload($payment, $successUrl, $cancelUrl);
    $response = $client->initSession($payload);
    if (!$response) {
      return NULL;
    }

    return $response[ResponseFields::REDIRECT_URL->value];
  }

  /**
   * Build the payload to send to the RagaPay API.
   */
  protected function buildSessionPayload(PaymentInterface $payment, string $successUrl, string $cancelUrl): array {
    $order = $payment->getOrder();
    $customer = $order->getCustomer();
    $amount = $payment->getAmount();
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $paymentGatewayPlugin */
    $paymentGatewayPlugin = $payment->getPaymentGateway()->getPlugin();
    $password = $paymentGatewayPlugin->getConfiguration()[RagaPayOffsite::PASSWORD_CONFIG];
    $payload = [
      PayloadFields::MERCHANT_KEY->value => $paymentGatewayPlugin->getConfiguration()[RagaPayOffsite::MERCHANT_CONFIG],
      PayloadFields::OPERATION->value => Operations::PURCHASE->value,
      PayloadFields::METHODS->value => [
        Methods::CARD->value,
      ],
      PayloadFields::PARAMETERS->value => [
        Methods::CARD->value => [
          'bnrg_installm_def' => '00',
          'bnrg_installm_months' => '03',
          'bnrg_installm_plan' => '03',
        ],
      ],
      PayloadFields::ORDER->value => [
        PayloadFields::ORDER_NUMBER->value => $order->id(),
        PayloadFields::ORDER_AMOUNT->value => PriceFormatter::formatNumber($amount),
        PayloadFields::ORDER_CURRENCY->value => $amount->getCurrencyCode(),
        PayloadFields::ORDER_DESCRIPTION->value => $order->label(),
      ],
      PayloadFields::SUCCESS_URL->value => $successUrl,
      PayloadFields::CANCEL_URL->value => $cancelUrl,
      PayloadFields::CUSTOMER->value => [
        PayloadFields::CUSTOMER_NAME->value => $customer->getDisplayName(),
        PayloadFields::CUSTOMER_EMAIL->value => $order->getEmail(),
      ],
      PayloadFields::HASH->value => HashBuilder::generateOrderSecureHash($payment, $password),
    ];
    $billingProfile  = $order->getBillingProfile();
    if ($billingProfile) {
      /** @var \Drupal\address\Plugin\Field\FieldType\AddressItem $billingAddress */
      $billingAddress = $billingProfile->get('address')->first();
      if ($billingAddress) {
        $billingAddressArray = [];
        if ($country = $billingAddress->getCountryCode()) {
          $billingAddressArray[PayloadFields::BILLING_ADDRESS_COUNTRY->value] = $country;
        }
        if ($state = $billingAddress->getAdministrativeArea()) {
          $billingAddressArray[PayloadFields::BILLING_ADDRESS_STATE->value] = $state;
        }
        if ($city = $billingAddress->getLocality()) {
          $billingAddressArray[PayloadFields::BILLING_ADDRESS_CITY->value] = $city;
        }
        if ($address = $billingAddress->getAddressLine1()) {
          $billingAddressArray[PayloadFields::BILLING_ADDRESS_LINE1->value] = $address;
        }
        if ($zip = $billingAddress->getPostalCode()) {
          $billingAddressArray[PayloadFields::BILLING_ADDRESS_ZIP->value] = $zip;
        }
        $payload[PayloadFields::BILLING_ADDRESS->value] = $billingAddressArray;
      }
    }

    return $payload;
  }
}
